using System;
using System.Text;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using javax.crypto;
using java.security;
using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Crypto.Engines;

namespace Proyecto
{
    /*
     * Clase Cabecera: Almacena un objeto InfoCabecera cifrado y representado en bytes, 
     * acompaado de su resumen (antes del cifrado).
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    [Serializable]
    class Cabecera
    {
        internal byte[] bytes;  //guarda un objeto InfoCabecera cifrado y en bytes
        internal byte[] resumen; //el resumen de bytes antes de cifrar, es decir, del objeto InfoCabecera

        //instancia un objeto cabecera cifrando la informacin con RSA
        public Cabecera(InfoCabecera info, PublicKey clavePublicaRSA)
        {
            byte[] bytes_info = info.aBytes();

            Cipher cipher = Cipher.getInstance("RSA");
            cipher.init(Cipher.ENCRYPT_MODE, clavePublicaRSA);

            try
            {
                bytes = cipher.doFinal(bytes_info);

                MessageDigest md = MessageDigest.getInstance("SHA-1");
                resumen = md.digest(bytes_info);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.GetType());
                Console.WriteLine(e.Message);
            }
        }

        //instancia un objeto cabecera cifrando la informacin con ElGamal
        public Cabecera(InfoCabecera info, AsymmetricKeyParameter clavePublicaElGamal)
        {
            byte[] bytes_info = info.aBytes();

            ElGamalEngine cipher = new ElGamalEngine();
            cipher.Init(true, clavePublicaElGamal); //true = cifrar

            try
            {
                bytes = cipher.ProcessBlock(bytes_info, 0, bytes_info.Length);

                MessageDigest md = MessageDigest.getInstance("SHA-1");
                resumen = md.digest(bytes_info);
            }
            catch (Exception)
            {
                throw;
            }

        }

        internal byte[] Bytes
        {
            get
            {
                return bytes;
            }
        }

        internal byte[] Resumen
        {
            get
            {
                return resumen;
            }
        }

        //comprueba la cabecera con una clave privada RSA
        internal InfoCabecera comprobarCabecera(PrivateKey clavePrivada)
        {
            Cipher cipher = Cipher.getInstance("RSA");
            cipher.init(Cipher.DECRYPT_MODE, clavePrivada);

            try
            {
                byte[] descifrado = cipher.doFinal(bytes);

                MessageDigest md = MessageDigest.getInstance("SHA-1");
                byte[] resumen_aux = md.digest(descifrado);

                //si la clave es vlida para la cabecera recuperamos
                //el objeto InfoCabecera a partir de los bytes descifrados
                if (comparar_vectores(resumen, resumen_aux))
                {
                    MemoryStream memStream = new MemoryStream();
                    BinaryFormatter formatter = new BinaryFormatter();

                    memStream.Write(descifrado, 0, descifrado.Length);
                    memStream.Seek(0, SeekOrigin.Begin);

                    InfoCabecera infoCabecera = recuperarInfo(memStream.ToArray());
                    return infoCabecera;
                }
                else
                {
                    Console.WriteLine("Los resumenes no coinciden");
                    return null;
                }
            }
            //esta excepcin es lanzada por Cipher.doFinal()
            //si la clave no puede descifrar la informacin
            catch (BadPaddingException)
            {
                return null;
            }
            catch (Exception oe)
            {
                Console.WriteLine(oe.Message);
                return null;
            }            
        }

        //comprueba la cabecera con una clave privada ElGamal
        internal InfoCabecera comprobarCabecera(AsymmetricKeyParameter clavePrivadaElGamal)
        {
            ElGamalEngine cipher = new ElGamalEngine();
            cipher.Init(false, clavePrivadaElGamal); //false = descifrar

            try
            {
                byte[] descifrado = cipher.ProcessBlock(bytes, 0, bytes.Length);                

                MessageDigest md = MessageDigest.getInstance("SHA-1");
                byte[] resumen_aux = md.digest(descifrado);

                //si la clave es vlida para la cabecera recuperamos
                //el objeto InfoCabecera a partir de los bytes descifrados
                if (comparar_vectores(resumen, resumen_aux))
                {
                    MemoryStream memStream = new MemoryStream();
                    BinaryFormatter formatter = new BinaryFormatter();

                    memStream.Write(descifrado, 0, descifrado.Length);
                    memStream.Seek(0, SeekOrigin.Begin);

                    InfoCabecera infoCabecera = recuperarInfo(memStream.ToArray());
                    return infoCabecera;
                }
                else
                {
                    Console.WriteLine("Los resumenes no coinciden");
                    return null;
                }
            }
            //esta excepcin se lanza si Cipher.doFinal() no puede
            //descifrar la informacin
            catch (BadPaddingException)
            {
                return null;
            }
            catch (Exception oe)
            {
                Console.WriteLine(oe.Message);
                return null;
            }
        }

        static internal Boolean comparar_vectores(byte[] v1, byte[] v2)
        {
            if (v1.Length != v2.Length)
                return false;
            for (int i = 0; i < v1.Length; i++)
            {
                if (v1[i].Equals(v2[i]) == false)
                    return false;
            }
            return true;
        }

        //recupera la informacin contenida en el array de bytes en claro
        //es un proceso similar a la deserializacin
        internal static InfoCabecera recuperarInfo(byte[] array)
        {
            StringBuilder sb = new StringBuilder();
            string nombre;
            string algoritmo;
            string paramsAlgoritmo;
            byte[] claveSimetrica;
            int pos = 1;
            int i = 0;

            //nombre
            while (array[pos] != (byte)'\0')
            {
                sb.Append((char)array[pos]);
                pos++;
            }
            nombre = sb.ToString();
            //Console.WriteLine("nombre: "+nombre);
            
            //algoritmo
            sb = new StringBuilder();
            pos = pos + 2; //saltamos dos: '\0' y el primer byte de la siguiente cadena (indica el nmero de caracteres de sta)
            while (array[pos] != (byte)'\0')
            {
                sb.Append((char)array[pos]);
                pos++;
            }
            algoritmo = sb.ToString();

            //paramsAlgoritmo
            sb = new StringBuilder();
            pos = pos + 2; //saltamos dos: '\0' y el primer byte de la siguiente cadena (indica el nmero de caracteres de sta)
            while (array[pos] != (byte)'\0')
            {
                sb.Append((char)array[pos]);
                pos++;
            }
            paramsAlgoritmo = sb.ToString();

            pos++; //la clave no es un string, as que no tiene un byte inicial indicando el n de caracteres, slo hay que saltar el '\0'
            claveSimetrica = new byte[array.Length-1-pos];
            while (array[pos] != (byte)'\0')
            {
                claveSimetrica[i] = array[pos];
                pos++;
                i++;
            }

            return new InfoCabecera(nombre, algoritmo, paramsAlgoritmo, claveSimetrica);
        }
    }
}