using System;
using System.Collections;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using javax.crypto;
using java.security;
using Org.BouncyCastle.Crypto;

namespace Proyecto
{
    /*
     * Clase Cabeceras: contenedor para una lista de objetos Cabecera.
     * Se encarga de gestionarla, cifrarla, descifrarla, serializarla y deserializarla
     * Autor: Juan lvaro Muoz Naranjo
     * Fecha: Junio de 2007
     */
    [Serializable]
    class Cabeceras
    {
        private ArrayList lista;

        internal Cabeceras()
        {
            lista = new ArrayList();
        }

        internal Cabeceras(ArrayList lista)
        {
            this.lista = lista;
        }

        internal int Tamao
        {
            get
            {
                return lista.Count;
            }
        }

        //busca en la lista de cabeceras una que pueda descifrarse con la clave privada proporcionada
        //es decir, una cabecera que sea nuestra, para proporcionarnos acceso al fichero
        //devuelve un objeto InfoCabecera si se encontr una que pudie descifrarse con nuestra clave privada
        //devuelve null si no se encontr ninguna (no tenemos permiso para ver el fichero)
        internal InfoCabecera buscarCabecera(PrivateKey clavePrivadaRSA)
        {
            Cabecera cabecera;
            InfoCabecera info;

            foreach (Object o in lista)
            {
                cabecera = (Cabecera)o;
                info = cabecera.comprobarCabecera(clavePrivadaRSA);
                if (info != null)
                    return info;
            }
            return null;
        }

        //busca una cabecera usando ElGamal
        //devuelve el objeto InfoCabecera si se encontr
        //devuelve null si no se encontr ninguna que pudiese descifrarse con la clave privada
        internal InfoCabecera buscarCabecera(AsymmetricKeyParameter clavePrivadaElGamal)
        {
            Cabecera cabecera;
            InfoCabecera info;

            foreach (Object o in lista)
            {
                cabecera = (Cabecera)o;
                info = cabecera.comprobarCabecera(clavePrivadaElGamal);
                if (info != null)
                    return info;
            }
            return null;
        }

        //aade una cabecera a la lista cifrndola con RSA
        internal void aadirCabecera(InfoCabecera info, PublicKey clavePublicaRSA)
        {
            lista.Add(new Cabecera(info, clavePublicaRSA));
        }

        //aade una cabecera a la lista cifrndola con ElGamal
        internal void aadirCabecera(InfoCabecera info, AsymmetricKeyParameter clavePublicaElGamal)
        {
            lista.Add(new Cabecera(info, clavePublicaElGamal));
        }

        //aade una cabecera a la lista cifrndola con RSA
        internal void aadirCabecera(string nombre, string algoritmoSimetrico, string paramsAlgoritmo, byte[] claveSimetrica,
            PublicKey clavePublicaRSA)
        {
            lista.Add(new Cabecera(new InfoCabecera(nombre, algoritmoSimetrico, paramsAlgoritmo, claveSimetrica), 
                clavePublicaRSA));
        }

        //aade una cabecera a la lista cifrndola con ElGamal
        internal void aadirCabecera(string nombre, string algoritmoSimetrico, string paramsAlgoritmo, byte[] claveSimetrica,
            AsymmetricKeyParameter clavePublicaElGamal)
        {
            lista.Add(new Cabecera(new InfoCabecera(nombre, algoritmoSimetrico, paramsAlgoritmo, claveSimetrica),
                clavePublicaElGamal));
        }

        //serializa el objeto en un vector de bytes
        internal byte[] aBytes()
        {
            MemoryStream memStream = new MemoryStream();
            BinaryFormatter formatter = new BinaryFormatter();

            formatter.Serialize(memStream, this);
            memStream.Close();

            return memStream.ToArray();
        }

        //deserializa el objeto de un vector de bytes
        internal static Cabeceras recuperarCabeceras(byte[] buffer)
        {
            MemoryStream memStream = new MemoryStream(buffer);
            BinaryFormatter formatter = new BinaryFormatter();

            Cabeceras cabeceras = (Cabeceras)formatter.Deserialize(memStream);
            memStream.Close();

            return cabeceras;
        }

        //deserializa el objeto de un stream
        internal static Cabeceras recuperarCabeceras(Stream el_stream)
        {
            BinaryFormatter formatter = new BinaryFormatter();

            return (Cabeceras)formatter.Deserialize(el_stream);
        }
    }
}
